﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.47
@EndOfDesignText@
#if B4A
'ignore DIP related warnings as they are not relevant when working with BitmapCreator.
#IgnoreWarnings: 6
#end if
Sub Class_Globals
	Type myUD(isTouched As Boolean, force As B2Vec2, switchStatus As Int)
	Public X2 As X2Utils
	Private xui As XUI 'ignore
	Public world As B2World
	Public Ground As X2BodyWrapper
	Private ivForeground As B4XView
 	Public TileMap As X2TileMap
	Public Const ObjectLayer As String = "Object Layer 1"
	Public RightDown, LeftDown, FireDown As Boolean
	Private Const ConfigFile As String = "config.txt" 'ignore 
	Private PanelForTouch As B4XView
	Private states As Label
	Type SwitchData (State As Boolean)
	Type BallData (Force As B2Vec2)
	Public lblStats As Label
End Sub

Public Sub Initialize (Parent As B4XView)
	Parent.LoadLayout("1")
	world.Initialize("world", world.CreateVec2(0, -9.8)) 'no gravity
	X2.Initialize(Me, ivForeground, world)
	Dim WorldHeight As Float = 6 
	Dim WorldWidth As Float = WorldHeight * 1.3333
	X2.ConfigureDimensions(world.CreateVec2(WorldWidth / 2, WorldHeight / 2), WorldWidth)
 
	'comment to disable debug drawing
	'X2.EnableDebugDraw
End Sub

Public Sub StartGame
	If X2.IsRunning Then Return
	X2.Reset
  
	X2.UpdateWorldCenter(X2.CreateVec2(X2.ScreenAABB.Width / 2, X2.ScreenAABB.Height / 2))
	TileMap.Initialize(X2, File.DirAssets, "map.json", ivForeground)
	Dim TileSizeMeters As Float = X2.ScreenAABB.Height / TileMap.TilesPerColumn
	TileMap.SetSingleTileDimensionsInMeters(TileSizeMeters, TileSizeMeters)
	TileMap.PrepareObjectsDef(ObjectLayer)	
	loadGroundBodies 'create bodies
	X2.Start
End Sub

Sub loadGroundBodies
	Dim ol As X2ObjectsLayer = TileMap.Layers.Get(ObjectLayer)
	For Each template As X2TileObjectTemplate In ol.ObjectsById.Values
		Select template.Name
			Case "gear"
				Dim gearbody As X2BodyWrapper = TileMap.CreateObject(template)
				gearbody.Body.AngularVelocity = 1 'set angular velocity for kinematic body
			Case "switch"
				Dim switch As X2BodyWrapper = TileMap.CreateObject(template)
				Dim sd As SwitchData
				sd.Initialize
				switch.Tag = sd
			Case "ball"
				CreateBall(template)
			Case Else 
				TileMap.CreateObject(template)	
		End Select
	Next
End Sub

Private Sub CreateBall(template As X2TileObjectTemplate)
	Dim ball As X2BodyWrapper = TileMap.CreateObject(template)
	Dim bd As BallData
	bd.Initialize
	ball.Tag = bd
End Sub
 
Public Sub Resize
	X2.ImageViewResized
End Sub

Public Sub Tick (GS As X2GameStep)	
	updateBallForce 'apply force if hit sensor
	writeStates
End Sub

Sub updateBallForce
	For Each body As B2Body In world.AllBodies
		Dim bw As X2BodyWrapper = body.Tag
		If bw.Tag Is BallData Then
			Dim bd As BallData = bw.Tag
			body.ApplyForce(bd.Force, body.WorldCenter)
		End If
	Next
End Sub
 
Public Sub DrawingComplete

End Sub

Sub writeStates
	states.Text = "Bodies in World: " & world.AllBodies.Size & "  FPS: " & NumberFormat2(X2.FPS,1,0,0,False)
End Sub
  
'This event fires while the world is locked.
'We need to use AddFutureTask to run code after the physics engine completed the time step.
Private Sub World_BeginContact (Contact As B2Contact)
	Dim bodies As X2BodiesFromContact = X2.GetBodiesFromContact(Contact,  "ball")
	If bodies <> Null Then
		If bodies.OtherBody.Name = "forceright" Then
			Dim bd As BallData = bodies.ThisBody.Tag
			bd.force.Set(0.1,0)
		else if bodies.OtherBody.Name = "forceup" Then
			Dim bd As BallData = bodies.ThisBody.Tag
			bd.force.Set(0,0.5)
		End If
	End If
End Sub
 
Private Sub World_EndContact(Contact As B2Contact)
	Dim bodies As X2BodiesFromContact = X2.GetBodiesFromContact(Contact,  "ball")
	If bodies <> Null Then
		If bodies.OtherBody.Name = "forceright" Or bodies.OtherBody.Name = "forceup" Then
			Dim bd As BallData = bodies.ThisBody.Tag
			bd.Force.Set(0, 0)
		End If
	End If
End Sub

 
Private Sub getVec(x As Float, y As Float) As B2Vec2 'ignore
	Dim v As B2Vec2
	v.Set(x,y)
	Return v 
End Sub

'Return True to stop the game loop
Public Sub BeforeTimeStep (GS As X2GameStep) As Boolean
	Return False
End Sub

Public Sub StopGame
	X2.Stop
End Sub
 
Private Sub PanelForTouch_Touch (Action As Int, X As Float, Y As Float)
	If Action = PanelForTouch.TOUCH_ACTION_DOWN Then
		Dim Switch As X2BodyWrapper = TouchedSwitch(X, Y)
		If Switch <> Null Then
			Log("YOU TOUCHED A SWITCH")
			Switch.CurrentFrame = 1
			Dim Sw As SwitchData = Switch.Tag
			Sw.State = Not(Sw.State)
			If Sw.State Then
				Switch.CurrentFrame = 1
			Else
				Switch.CurrentFrame = 0
			End If
		Else
			Dim template As X2TileObjectTemplate = TileMap.GetObjectTemplateByName(ObjectLayer, "ball")
			template.BodyDef.Position = X2.ScreenPointToWorld(X, Y)
			CreateBall(template)
		End If
	End If
End Sub


Sub TouchedSwitch(x As Float, y As Float) As X2BodyWrapper
	Dim aabb As B2AABB
	Dim vec As B2Vec2 = X2.ScreenPointToWorld(x, y)
	aabb.Initialize2(vec, vec)
	For Each body As B2Body In world.QueryAABBToMapOfBodies(aabb).Keys
		If body.FirstFixture.Shape.TestPoint(body.Transform, vec) Then
			Dim bw As X2BodyWrapper = body.Tag
			If bw.Tag Is SwitchData Then
				Return bw
			End If
		End If
	Next
	Return Null
End Sub
